<?php

namespace Drupal\vocabulary_clone\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\taxonomy\Entity\Vocabulary;
use Drupal\Core\Url;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Drupal\vocabulary_clone\Controller\VocabularyCloneController;

/**
 *
 */
class VocabularyCloneForm extends FormBase {

  protected $vocabulary;

  /**
   *
   */
  public function getFormId() {
    return 'vocabulary_clone_form';
  }

  /**
   *
   */
  public function buildForm(array $form, FormStateInterface $form_state, $vocabulary_id = NULL) {
    $this->vocabulary = Vocabulary::load($vocabulary_id);

    $form['new_machine_name'] = [
      '#type' => 'machine_name',
      '#title' => $this->t('New Machine Name'),
      '#description' => $this->t('Enter a unique machine name for the cloned vocabulary.'),
      '#required' => TRUE,
      '#machine_name' => [
        'exists' => [$this, 'vocabularyExists'],
      ],
    ];

    $form['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Clone Vocabulary'),
    ];

    return $form;
  }

  /**
   *
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $new_machine_name = $form_state->getValue('new_machine_name');
    if (Vocabulary::load($new_machine_name)) {
      $form_state->setErrorByName('new_machine_name', $this->t('The machine name is already in use.'));
    }
  }

  /**
   *
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $new_machine_name = $form_state->getValue('new_machine_name');

    // Create an instance of the controller class and call the clone method, passing the new machine name.
    $controller = new VocabularyCloneController();
    $response = $controller->cloneVocabulary($this->vocabulary->id(), $new_machine_name);

    if ($response instanceof RedirectResponse) {
      $form_state->setRedirectUrl(Url::fromRoute('entity.taxonomy_vocabulary.collection'));
    }
    else {
      $this->messenger()->addError($response->getContent());
    }
  }

  /**
   *
   */
  public function vocabularyExists($machine_name) {
    return Vocabulary::load($machine_name) !== NULL;
  }

}
